-- name: \\#ed6d28\\HL2 \\#ffffff\\HUD
-- description: A recreation of the HUD from \\#ed6d28\\HL2\\#ffffff\\, \nby \\#6269b5\\Birdekek \\#47ded1\\\n[HL1 + HL2 HUD Modpack]\n\n\\#ffffff\\This mod replaces the default SM64 HUD with a recreation of the HUD from\n\\#ed6d28\\Half-Life 2\\#ffffff\\, a gane from 2004 made by Valve.

TEX_DIS1 = get_texture_info("backgroundlines")
TEX_DIS2 = get_texture_info("backgroundlines2")
TEX_DIS3 = get_texture_info("backgroundlines3")
TEX_HUDCORNER1 = get_texture_info("cornertl")
TEX_HUDCORNER2 = get_texture_info("cornertr")
TEX_HUDCORNER3 = get_texture_info("cornerbl")
TEX_HUDCORNER4 = get_texture_info("cornerbr")
TEX_COIN = get_texture_info("coin")
TEX_STAR = get_texture_info("star")

SOUND_DEATH = audio_sample_load("death.ogg")

local yellowCol = {245, 234, 32}
local redCol = {204, 67, 67}
local healthcolorv = 1
local timer = 0
local displacement = 0

local function lerpColor(color1, color2, alpha)
    return {
        color1[1] + (color2[1] - color1[1]) * alpha,
        color1[2] + (color2[2] - color1[2]) * alpha,
        color1[3] + (color2[3] - color1[3]) * alpha
    }
end

local function update()
    local m = gMarioStates[0]
    local mathed_health = math.ceil((m.health - 255) * 100 / (2176 - 255))
    
    if mathed_health <= 25 then
        if healthcolorv ~= 0 then healthcolorv = healthcolorv - 0.1 end
        if healthcolorv <= 0 then healthcolorv = 0 end
    else
        if healthcolorv ~= 1 then healthcolorv = healthcolorv + 0.1 end
        if healthcolorv >= 1 then healthcolorv = 1 end
    end

    timer = timer + 0.25

    if timer == 256 then timer = 0 end
end

local function death()
    local m = gMarioStates[0]
    audio_sample_play(SOUND_DEATH, m.pos, (is_game_paused() and 0 or 1))
end

local function on_hud_render()
    local m = gMarioStates[0]
    local mathed_health = math.ceil((m.health - 255) * 100 / (2176 - 255))
    djui_hud_set_resolution(RESOLUTION_N64)
    local x = djui_hud_get_screen_width()
    local y = djui_hud_get_screen_height() - 8

    if displacement < 4 then displacement = displacement + 0.25 end
    if displacement >= 4.0 then displacement = 1 end

    -- HEALTH BOX

    djui_hud_set_adjusted_color(0, 0, 0, 75)

    if mathed_health <= 25 then
        djui_hud_set_adjusted_color((math.sin(timer) * ((-healthcolorv + 1) * 8) + 32), 0, 0, 75)
    end

    djui_hud_render_texture(TEX_HUDCORNER1, 8, y - 25, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER3, 8, y - 4, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER2, 84, y - 25, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER4, 84, y - 4, 0.25, 0.25)
    
    djui_hud_render_rect(12, y - 25, 72, 25)
    djui_hud_render_rect(8, y - 21, 4, 17)
    djui_hud_render_rect(84, y - 21, 4, 17)

    djui_hud_set_adjusted_color(0, 0, 0, 75)

    -- LIFE BOX

    djui_hud_render_texture(TEX_HUDCORNER1, x - 84, y - 25, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER3, x - 84, y - 4, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER2, x - 8, y - 25, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER4, x - 8, y - 4, 0.25, 0.25)
    
    djui_hud_render_rect(x - 80, y - 25, 72, 25)
    djui_hud_render_rect(x - 8, y - 21, 4, 17)
    djui_hud_render_rect(x - 84, y - 21, 4, 17)

    -- COIN BOX

    djui_hud_render_texture(TEX_HUDCORNER1, 8, y - 37, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER3, 8, y - 33, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER2, 32, y - 37, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER4, 32, y - 33, 0.25, 0.25)
    
    djui_hud_render_rect(12, y - 37, 20, 8)
    
    -- STAR BOX

    djui_hud_render_texture(TEX_HUDCORNER1, 8, y - 49, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER3, 8, y - 45, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER2, 32, y - 49, 0.25, 0.25)
    djui_hud_render_texture(TEX_HUDCORNER4, 32, y - 45, 0.25, 0.25)
    
    djui_hud_render_rect(12, y - 49, 20, 8)

    -- render the health stuff

    local colors = lerpColor(redCol, yellowCol, healthcolorv)

    djui_hud_set_adjusted_color(colors[1], colors[2], colors[3], 255)

    local measured_text = djui_hud_measure_text(tostring(mathed_health))

    djui_hud_set_font(FONT_NORMAL)
    djui_hud_print_text("HEALTH", 14, y - 12, 0.30)
    djui_hud_print_text(tostring(mathed_health), 63 - ((measured_text * 0.75) / 2), y - 24, 0.75)

    if math.floor(displacement) == 1 then
        djui_hud_render_texture(TEX_DIS1, 34, y - 24, 0.45, 0.35)
    elseif math.floor(displacement) == 2 then
        djui_hud_render_texture(TEX_DIS2, 34, y - 24, 0.45, 0.35)
    elseif math.floor(displacement) == 3 then
       djui_hud_render_texture(TEX_DIS3, 34, y - 24, 0.45, 0.35)
    end

    -- render the coin, star, and both of their texts as yellow

    djui_hud_set_adjusted_color(245, 234, 32, 255)

    djui_hud_render_texture(TEX_COIN, 10, y - 35, 0.25, 0.25)

    djui_hud_render_texture(TEX_STAR, 10, y - 47, 0.25, 0.25)

    djui_hud_print_text("x " .. tostring(m.numCoins), 17, y - 37, 0.25)

    djui_hud_print_text("x " .. tostring(m.numStars), 17, y - 49, 0.25)

    -- render life box text

    local measured_text2 = djui_hud_measure_text(tostring(m.numLives))

    djui_hud_set_font(FONT_NORMAL)
    djui_hud_print_text("LIVES", x - 76, y - 12, 0.30)
    djui_hud_print_text(tostring(m.numLives), (x - 30) - ((measured_text2 * 0.75) / 2), y - 24, 0.75)

    if math.floor(displacement) == 1 then
        djui_hud_render_texture(TEX_DIS1, x - 60, y - 24, 0.45, 0.35)
    elseif math.floor(displacement) == 2 then
        djui_hud_render_texture(TEX_DIS2, x - 60, y - 24, 0.45, 0.35)
    elseif math.floor(displacement) == 3 then
       djui_hud_render_texture(TEX_DIS3, x - 60, y - 24, 0.45, 0.35)
    end

    hud_hide()
end

function djui_hud_set_adjusted_color(r, g, b, a)
    local multiplier = 1
    if is_game_paused() then multiplier = 0.5 end
    djui_hud_set_color(r * multiplier, g * multiplier, b * multiplier, a)
end

hook_event(HOOK_ON_HUD_RENDER, on_hud_render)
hook_event(HOOK_UPDATE, update)
hook_event(HOOK_ON_DEATH, death)